;;;; avrforth
;;;;
;;;; public domain


.include "target/defines"

.listmac

;;; user defines

  .set ramend = $10ff

  .set return_stack_size = $80

  .set upinit = $200

  .set cpu_frequency = 3860000
  .set baud_rate = 19200
  ;.set baud_rate = 31250

  
;;; register defines

  ; z
  .def zh = r31
  .def zl = r30

  ; y : parameter stack pointer
  .def yh = r29
  .def yl = r28

  ; x
  .def xh = r27
  .def xl = r26

  ; top of stack register
  .def tosh = r25
  .def tosl = r24
  .def tos = r24
  
  ; temp registers
  .def temp3 = r19
  .def temp2 = r18
  .def temp1 = r17
  .def temp0 = r16

  ; zero register
  .def zeroh = r15
  .def zerol = r14
  .def zero = r14

  ; temp for shifting z
  .def ztemp = r6

  ; saved return address
  ; for context switches

  .def srh = r5
  .def srl = r4
  .def sr = r4

  ; user pointer
  .def runningh = r3
  .def runningl = r2
  .def running = r2


;;;; macros

;;; shift z left
  .macro szl
  clr ztemp
  lsl zl
  rol zh
  rol ztemp
  out rampz,ztemp
  .endm

;;; shift z right
  .macro szr
  in ztemp,rampz
  lsr ztemp
  ror zh
  ror zl
  .endm

  
.org $1

codemem:


.org nrww
rjmp reset

;;;; macro words

.include "macros/oneplus.asm"
.include "macros/oneminus.asm"

.include "macros/twostar.asm"
.include "macros/twoslash.asm"

;;;; words

.set link = $0
.set mlink = $0

;;; stack manipulations

.include "words/dup.asm" ;;;
.include "words/drop.asm" ;;;
.include "words/nip.asm" ;;;
.include "words/over.asm"
.include "words/rot.asm" ;;;
.include "words/swap.asm" ;;;

.include "words/spstore.asm" ;;;
.include "words/spfetch.asm" ;;;

;;; return stack manipulations

.include "words/tor.asm" ;;;
.include "words/rfrom.asm" ;;;
.include "words/rfetch.asm" ;;;

.include "words/rpstore.asm" ;;;
.include "words/rpfetch.asm" ;;;

;;; constants

.include "const.asm" ;;;

;;; user area variables

.include "user.asm" ;;;

;;; memory operators

.include "words/store.asm" ;;;
.include "words/cstore.asm" ;;;
.include "words/fetch.asm" ;;;
.include "words/cfetch.asm" ;;;
.include "words/ifetch.asm"
.include "words/istore.asm"
.include "words/efetch.asm" ;;;

;;; math

.include "words/minus.asm" ;;;
.include "words/plus.asm" ;;;
.include "words/plusstore.asm" ;;;

.include "words/umin.asm" ;;;
.include "words/umax.asm" ;;;

;;; logical

.include "words/and.asm" ;;;
.include "words/or.asm" ;;;
.include "words/swapbytes.asm"
.include "words/xor.asm" ;;;

;;; interpreter

.include "words/accept.asm" ;;;
.include "words/execute.asm"
.include "words/fetchexecute.asm" ;;;
.include "words/find.asm" ;;;
.include "words/lit.asm"
.include "words/word.asm" ;;;
.include "words/go.asm" ;;;
.include "words/tick.asm" ;;;
.include "words/forth.asm" ;;;
.include "words/macro.asm" ;;;
.include "words/words.asm" ;;;

;;; compiler

.include "words/colon.asm" ;;;
.include "words/rightbracket.asm" ;;;
.include "words/c.asm" ;;;
.include "words/semicolon.asm" ;;;
.include "words/doublesemicolon.asm" ;;;
.include "words/l.asm" ;;;
.include "words/commaxt.asm" ;;;

;;; dictionary

.include "words/allot.asm" ;;;
.include "words/iallot.asm" ;;;
.include "words/here.asm" ;;;
.include "words/ihere.asm" ;;;
.include "words/comma.asm" ;;;
.include "words/icomma.asm" ;;;
.include "words/latest.asm" ;;;

;;; i/o

.include "words/cr.asm" ;;;
.include "words/space.asm" ;;;
.include "words/type.asm" ;;;
.include "words/itype.asm" ;;;
.include "words/itypeprime.asm" ;;;
.include "words/xquote.asm" ;;;
.include "words/udot.asm" ;;;

;;; numeric input

.include "num.asm" ;;;

;;; alignment and portability

.include "words/cells.asm" ;;;
.include "words/icells.asm" ;;;

;;; comparison

.include "words/true.asm" ;;;
.include "words/cequal.asm" ;;;
.include "words/notequal.asm" ;;;

;;; control

.include "words/if.asm" ;;;
.include "words/then.asm" ;;;

;;; startup

.include "words/cold.asm" ;;;

;;; uart

.include "words/emit.asm"
.include "words/key.asm"


;;; main entry point

reset:

  ; init return stack pointer
  ldi temp0,high(ramend)
  out SPH,temp0
  ldi temp0,low(ramend)
  out SPL,temp0

  ; init parameter stack pointer
  ldi yh,high(ramend-return_stack_size+1)
  ldi yl,low(ramend-return_stack_size+1)

  ; init user area pointer
;  ldi temp1,high(upinit)
;  ldi temp0,low(upinit)
;  movw running,temp0

  ; set baud rate of uart
  ldi temp0,high((cpu_frequency/(baud_rate*16))-1)
  sts UBRR0H,temp0
  ldi temp0,low((cpu_frequency/(baud_rate*16))-1)
  out UBRR0L,temp0

  ; enable transmitter and receiver
  ldi temp0,(1<<TXEN)|(1<<RXEN)
  out UCSR0B,temp0

  ; init zeroes register
  clr zeroh
  clr zerol

  ; make portc all outputs and clear it
  ; stk500 leds light when 1 is output to pin
  ldi temp0,$ff
  out $14,temp0

  out $15,temp0

  rjmp cold



.set lastword = link
.set lastmacro = mlink

