;; i!
;;
;; ( x addr -- )
;;
;; store cell to instruction memory

.dw link
.set link = pc - 1
.db 'i','!',0

istore:
  rcall over
  rcall over
  rcall spmbuf
  dup_
  rcall pageload
  dup_
  rcall spmerase
  dup_
  rcall spmwrite
  rcall spmrww
  drop_
  drop_
  ret
  

;; pageload
;;
;; ( addr -- )

pageload:
  ; save complete address for later
  dup_
  rcall tor

  ; get beginning of page
  rcall lit
  .dw pagemask
  rcall andd

  ; loop counter (in words)
  rcall lit
  .dw pagesize
  rcall swapp

pageload_loop:
  rcall over
  adiw tos,0
  drop_
  breq pageload_done

  ; don't update temp buffer if it's the
  ; value we're changing. each location
  ; can only be written once.
  dup_
  rcall rfetch
  rcall notequal
  adiw tos,0
  drop_
  breq pageload_dontcopy

pageload_copy:
  dup_
  rcall ifetch
  rcall over
  rcall spmbuf

pageload_dontcopy:
  oneplus_
  rcall swapp
  oneminus_
  rcall swapp
  rjmp pageload_loop

pageload_done:
  drop_
  drop_
  rcall rfrom
  drop_
  ret


;; dospm
;;
;; ( spmcsr x addr -- )
;;
;; execute spm instruction

.dw link
.set link = pc - 1
.db 'd','o','s','p','m',0

dospm:
  ; disable interrupts if enabled,
  ; store status register
  in temp0,SREG
  cli

  movw z,tos
  szl
  drop_

  movw r0,tos
  drop_

  sts SPMCSR,tosl
  spm
  drop_

dospm_wait:
  ; wait until done
  lds temp1,SPMCSR
  sbrc temp1,SPMEN
  rjmp dospm_wait

  ; restore status register
  out SREG,temp0
  ret


;; spmbuf
;;
;; ( x addr -- )

.dw link
.set link = pc - 1
.db 's','p','m','b','u','f',0

spmbuf:
  rcall tor
  rcall tor
  rcall lit
  .dw (1<<SPMEN)
  rcall rfrom
  rcall rfrom
  rcall dospm
  ret

;; spmerase
;;
;; ( addr -- )

.dw link
.set link = pc - 1
.db 's','p','m','e','r','a','s','e',0

spmerase:
  rcall lit
  .dw pagemask
  rcall andd

  rcall tor
  rcall lit
  .dw (1<<PGERS|1<<SPMEN)
  rcall lit
  .dw 0
  rcall rfrom
  rcall dospm
  ret

;; spmwrite
;;
;; ( addr -- )

.dw link
.set link = pc - 1
.db 's','p','m','w','r','i','t','e',0

spmwrite:
  rcall lit
  .dw pagemask
  rcall andd
  
  rcall tor
  rcall lit
  .dw (1<<PGWRT|1<<SPMEN)
  rcall lit
  .dw 0
  rcall rfrom
  rcall dospm
  ret

;; spmrww
;;
;; ( -- )

.dw link
.set link = pc - 1
.db 's','p','m','r','w','w',0

spmrww:
  rcall lit
  .dw (1<<RWWSRE|1<<SPMEN)
  rcall lit
  .dw 0
  rcall lit
  .dw 0
  rcall dospm
  ret

